@@ -249,7 +249,7 @@ public:
 
     /// Default construction initializes to NaN when debugging but
     /// is left uninitialized otherwise.
-	SymMat(){ 
+    SymMat(){ 
     #ifndef NDEBUG
         setToNaN();
     #endif
@@ -741,7 +741,7 @@ public:
     /// Return true if no element contains an Infinity or a NaN.
     bool isFinite() const {return getAsVec().isFinite();}
 
-    /// For approximate comparisions, the default tolerance to use for a matrix is
+    /// For approximate comparisons, the default tolerance to use for a matrix is
     /// its shortest dimension times its elements' default tolerance.
     static double getDefaultTolerance() {return M*CNT<ELT>::getDefaultTolerance();}
 
@@ -958,14 +958,6 @@ template <int M, class E, int S> inline
 typename SymMat<M,E,S>::template Result<double>::Mul
 operator*(const double& l, const SymMat<M,E,S>& r) {return r*l;}
 
-template <int M, class E, int S> inline
-typename SymMat<M,E,S>::template Result<long double>::Mul
-operator*(const SymMat<M,E,S>& l, const long double& r)
-  { return SymMat<M,E,S>::template Result<long double>::MulOp::perform(l,r); }
-template <int M, class E, int S> inline
-typename SymMat<M,E,S>::template Result<long double>::Mul
-operator*(const long double& l, const SymMat<M,E,S>& r) {return r*l;}
-
 // m = m*int, int*m -- just convert int to m's precision float
 template <int M, class E, int S> inline
 typename SymMat<M,E,S>::template Result<typename CNT<E>::Precision>::Mul
@@ -1025,15 +1017,6 @@ typename CNT<double>::template Result<SymMat<M,E,S> >::Dvd
 operator/(const double& l, const SymMat<M,E,S>& r)
   { return CNT<double>::template Result<SymMat<M,E,S> >::DvdOp::perform(l,r); }
 
-template <int M, class E, int S> inline
-typename SymMat<M,E,S>::template Result<long double>::Dvd
-operator/(const SymMat<M,E,S>& l, const long double& r)
-  { return SymMat<M,E,S>::template Result<long double>::DvdOp::perform(l,r); }
-template <int M, class E, int S> inline
-typename CNT<long double>::template Result<SymMat<M,E,S> >::Dvd
-operator/(const long double& l, const SymMat<M,E,S>& r)
-  { return CNT<long double>::template Result<SymMat<M,E,S> >::DvdOp::perform(l,r); }
-
 // m = m/int, int/m -- just convert int to m's precision float
 template <int M, class E, int S> inline
 typename SymMat<M,E,S>::template Result<typename CNT<E>::Precision>::Dvd
@@ -1095,14 +1078,6 @@ template <int M, class E, int S> inline
 typename SymMat<M,E,S>::template Result<double>::Add
 operator+(const double& l, const SymMat<M,E,S>& r) {return r+l;}
 
-template <int M, class E, int S> inline
-typename SymMat<M,E,S>::template Result<long double>::Add
-operator+(const SymMat<M,E,S>& l, const long double& r)
-  { return SymMat<M,E,S>::template Result<long double>::AddOp::perform(l,r); }
-template <int M, class E, int S> inline
-typename SymMat<M,E,S>::template Result<long double>::Add
-operator+(const long double& l, const SymMat<M,E,S>& r) {return r+l;}
-
 // m = m+int, int+m -- just convert int to m's precision float
 template <int M, class E, int S> inline
 typename SymMat<M,E,S>::template Result<typename CNT<E>::Precision>::Add
@@ -1159,15 +1134,6 @@ typename CNT<double>::template Result<SymMat<M,E,S> >::Sub
 operator-(const double& l, const SymMat<M,E,S>& r)
   { return CNT<double>::template Result<SymMat<M,E,S> >::SubOp::perform(l,r); }
 
-template <int M, class E, int S> inline
-typename SymMat<M,E,S>::template Result<long double>::Sub
-operator-(const SymMat<M,E,S>& l, const long double& r)
-  { return SymMat<M,E,S>::template Result<long double>::SubOp::perform(l,r); }
-template <int M, class E, int S> inline
-typename CNT<long double>::template Result<SymMat<M,E,S> >::Sub
-operator-(const long double& l, const SymMat<M,E,S>& r)
-  { return CNT<long double>::template Result<SymMat<M,E,S> >::SubOp::perform(l,r); }
-
 // m = m-int, int-m // just convert int to m's precision float
 template <int M, class E, int S> inline
 typename SymMat<M,E,S>::template Result<typename CNT<E>::Precision>::Sub
