@@ -9,7 +9,7 @@
  * Biological Structures at Stanford, funded under the NIH Roadmap for        *
  * Medical Research, grant U54 GM072970. See https://simtk.org/home/simbody.  *
  *                                                                            *
- * Portions copyright (c) 2009-12 Stanford University and the Authors.        *
+ * Portions copyright (c) 2009-15 Stanford University and the Authors.        *
  * Authors: Michael Sherman                                                   *
  * Contributors:                                                              *
  *                                                                            *
@@ -27,9 +27,9 @@
 #include "SimTKcommon/basics.h"
 #include "SimTKcommon/Simmatrix.h"
 #include "SimTKcommon/internal/Random.h"
+#include "SimTKcommon/internal/Timing.h"
 
 #include <cmath>     
-#include <ctime>
 #include <algorithm> 
 #include <iostream>
 
@@ -57,11 +57,9 @@ namespace SimTK {
  *      - convenient testing of required argument checking (i.e., test
  *          fails unless an exception is thrown)
  *
- * The Testing.h header file is
- * <em>not</em> automatically included with SimTKcommon.h; you have to
- * ask for it explicilty. Here's how you use this facility:
+ * Here's how you use this facility:
  * <pre>
- * \#include "SimTKcommon/Testing.h"
+ * \#include "SimTKcommon.h"
  * void myFirstSubtest() {...}
  * void myNextSubtest() {...}
  * int main() {
@@ -114,6 +112,12 @@ namespace SimTK {
  *      SimTK_TEST_MUST_THROW_EXC(statement, exception) -- we expect a particular exception type
  *      SimTK_TEST_MUST_THROW_DEBUG(statement)  -- same as above but only checked in Debug builds
  *      SimTK_TEST_MUST_THROW_EXC_DEBUG(statement, exception) -- ditto
+ *
+ *      \#define SimTK_TEST_SUPPRESS_MUST_THROW 
+ *          -- Define this temporarily at top of test programs to make it 
+ *             easier to locate an *unexpected* exception while debugging. It
+ *             simply disables the "MUST_THROW" macros so you don't have to wade
+ *             through them in the debugger to get to the actual problem.
  * </pre>
  * The SimTK_TEST_EQ macros test scalar and composite numerical values for
  * equality to within a numerical tolerance, using both relative
@@ -159,14 +163,22 @@ namespace SimTK {
 class Test {
 public:
     class Subtest;
-    Test(const std::string& name) : testName(name)
+    Test(const std::string& name)
+    :   startCpuTime(SimTK::cpuTime()),
+        startRealTime(SimTK::realTime()),
+        testName(name)
     {
         std::clog << "Starting test " << testName << " ...\n";
-        startTime = std::clock(); 
     }
     ~Test() {
-        std::clog << "Done. " << testName << " time: " 
-                  << 1000*(std::clock()-startTime)/CLOCKS_PER_SEC << "ms.\n";
+        const double finalRealTime=SimTK::realTime();
+        const double finalCpuTime=SimTK::cpuTime();
+        std::ostringstream fmt;
+        fmt << std::fixed << std::setprecision(1);
+        fmt << "\n" << testName << " done." 
+            << " real/CPU ms: " << (finalRealTime-startRealTime)*1000
+            << " / "  << (finalCpuTime-startCpuTime)*1000 <<std::endl;
+        std::clog << fmt.str();
     }
 
     template <class T>
@@ -189,10 +201,6 @@ public:
         const double scale = n*std::max(std::max(std::abs(v1), std::abs(v2)), 1.0);
         return std::abs(v1-v2) < scale*(double)tol;
     }
-    static bool numericallyEqual(long double v1, long double v2, int n, double tol=defTol<long double>()) {
-        const long double scale = n*std::max(std::max(std::abs(v1), std::abs(v2)), 1.0l);
-        return std::abs(v1-v2) < scale*(long double)tol;
-    }
 
     // For integers we ignore tolerance.
     static bool numericallyEqual(int i1, int i2, int n, double tol=0) {return i1==i2;}
@@ -203,14 +211,6 @@ public:
     {   return numericallyEqual((double)v1, v2, n, tol); }
     static bool numericallyEqual(double v1, float v2, int n, double tol=defTol<float>())
     {   return numericallyEqual(v1, (double)v2, n, tol); }
-    static bool numericallyEqual(float v1, long double v2, int n, double tol=defTol<float>())
-    {   return numericallyEqual((long double)v1, v2, n, tol); }
-    static bool numericallyEqual(long double v1, float v2, int n, double tol=defTol<float>())
-    {   return numericallyEqual(v1, (long double)v2, n, tol); }
-    static bool numericallyEqual(double v1, long double v2, int n, double tol=defTol<double>())
-    {   return numericallyEqual((long double)v1, v2, n, tol); }
-    static bool numericallyEqual(long double v1, double v2, int n, double tol=defTol<double>())
-    {   return numericallyEqual(v1, (long double)v2, n, tol); }
 
     // Mixed int/floating just upgrades int to floating type.
     static bool numericallyEqual(int i1, float f2, int n, double tol=defTol<float>())
@@ -229,14 +229,6 @@ public:
     {   return numericallyEqual((double)i1,f2,n,tol); }
     static bool numericallyEqual(double f1, unsigned i2, int n, double tol=defTol<double>())
     {   return numericallyEqual(f1,(double)i2,n,tol); }
-    static bool numericallyEqual(int i1, long double f2, int n, double tol=defTol<long double>())
-    {   return numericallyEqual((long double)i1,f2,n,tol); }
-    static bool numericallyEqual(long double f1, int i2, int n, double tol=defTol<long double>())
-    {   return numericallyEqual(f1,(long double)i2,n,tol); }
-    static bool numericallyEqual(unsigned i1, long double f2, int n, double tol=defTol<long double>())
-    {   return numericallyEqual((long double)i1,f2,n,tol); }
-    static bool numericallyEqual(long double f1, unsigned i2, int n, double tol=defTol<long double>())
-    {   return numericallyEqual(f1,(long double)i2,n,tol); }
 
     template <class P>
     static bool numericallyEqual(const std::complex<P>& v1, const std::complex<P>& v2, int n, double tol=defTol<P>()) {
@@ -418,29 +410,35 @@ public:
         return Transform(randRotation(), randVec3());
     }
 private:
-    std::clock_t startTime;
+    const double startCpuTime;
+    const double startRealTime;
     std::string  testName;
 };
 
 /// Internal utility class for generating test messages for subtests.
 class Test::Subtest {
 public:
-    Subtest(const std::string& name) : subtestName(name)
+    Subtest(const std::string& name) 
+    :   startCpuTime(SimTK::cpuTime()),
+        startRealTime(SimTK::realTime()),
+        subtestName(name)
     {
-        char padded[128];
-        sprintf(padded, "%-20s", name.c_str());
-        paddedName = std::string(padded);
-        std::clog << "  " << paddedName << " ... " << std::flush;
-        startTime = std::clock(); 
+        std::clog << "  " << subtestName << " ...\n" << std::flush;
     }
     ~Subtest() {
-        std::clog << "done. " << paddedName << " time: " 
-                  << 1000*(std::clock()-startTime)/CLOCKS_PER_SEC << "ms.\n";
+        const double finalRealTime=SimTK::realTime();
+        const double finalCpuTime=SimTK::cpuTime();
+        std::ostringstream fmt;
+        fmt << std::fixed << std::setprecision(1);
+        fmt << "  " << subtestName << " done."
+            << " real/CPU ms: " << (finalRealTime-startRealTime)*1000
+            << " / "  << (finalCpuTime-startCpuTime)*1000 <<std::endl;
+        std::clog << fmt.str();
     }
 private:
-    std::clock_t startTime;
+    const double startCpuTime;
+    const double startRealTime;
     std::string  subtestName;
-    std::string  paddedName; // name plus some blanks
 };
 
 } // namespace SimTK
@@ -541,6 +539,8 @@ private:
     {SimTK_ASSERT1_ALWAYS(!SimTK::Test::numericallyEqual((v1),(v2),1,(tol)),   \
      "Test values should NOT have been numerically equivalent at tolerance=%g.",(tol));}
 
+#ifndef SimTK_TEST_SUPPRESS_EXPECTED_THROW
+
 /// Test that the supplied statement throws an std::exception of some kind.
 #define SimTK_TEST_MUST_THROW(stmt)             \
     do {int threw=0; try {stmt;}                \
@@ -550,6 +550,17 @@ private:
         if (threw==2) SimTK_TEST_FAILED1("Expected statement\n%s\n  to throw an std::exception but it threw something else.",#stmt); \
     }while(false)
 
+/// Test that the supplied statement throws an std::exception of some kind, and
+/// show what message got thrown.
+#define SimTK_TEST_MUST_THROW_SHOW(stmt)        \
+    do {int threw=0; try {stmt;}                \
+        catch(const std::exception& e) {threw=1; \
+            std::cout << "(OK) Threw: " << e.what() << std::endl;}  \
+        catch(...){threw=2;}                    \
+        if (threw==0) SimTK_TEST_FAILED1("Expected statement\n----\n%s\n----\n  to throw an exception but it did not.",#stmt); \
+        if (threw==2) SimTK_TEST_FAILED1("Expected statement\n%s\n  to throw an std::exception but it threw something else.",#stmt); \
+    }while(false)
+
 /// Test that the supplied statement throws a particular exception.
 #define SimTK_TEST_MUST_THROW_EXC(stmt,exc)     \
     do {int threw=0; try {stmt;}                \
@@ -577,7 +588,7 @@ private:
 
 // When we're only required to throw in Debug, we have to suppress the
 // test case altogether in Release because it may cause damage. 
-#if defined(NDEBUG)
+#ifdef NDEBUG
     /// Include a bad statement when in Debug and insist that it get caught,
     /// but don't include the statement at all in Release.
     #define SimTK_TEST_MUST_THROW_DEBUG(stmt)
@@ -594,7 +605,15 @@ private:
                 SimTK_TEST_MUST_THROW_EXC(stmt,exc)
 #endif
 
-
+#else // expected throws are suppressed
+#define SimTK_TEST_MUST_THROW(stmt)
+#define SimTK_TEST_MUST_THROW_SHOW(stmt)
+#define SimTK_TEST_MUST_THROW_EXC(stmt,exc)
+#define SimTK_TEST_MAY_THROW(stmt)
+#define SimTK_TEST_MAY_THROW_EXC(stmt,exc)
+#define SimTK_TEST_MUST_THROW_DEBUG(stmt)
+#define SimTK_TEST_MUST_THROW_EXC_DEBUG(stmt,exc)
+#endif
 
 
 //  End of Regression testing group.
