@@ -32,6 +32,7 @@
 #include "simmath/Differentiator.h"
 #include "simmath/internal/ContactGeometry.h"
 
+#include <atomic>
 #include <limits>
 
 namespace SimTK {
@@ -64,7 +65,7 @@ public:
     (thread safe). Each distinct type of ContactGeometry should use this to
     initialize a static variable for that concrete class. */
     static ContactGeometryTypeId  createNewContactGeometryTypeId()
-    {   static AtomicInteger nextAvailableId = 1;
+    {   static std::atomic<int> nextAvailableId(1);
         return ContactGeometryTypeId(nextAvailableId++); }
 
     virtual ContactGeometryImpl*  clone() const = 0;
@@ -92,17 +93,17 @@ public:
     virtual void calcCurvature(const Vec3& point, Vec2& curvature, 
                        Rotation& orientation) const
     {   SimTK_THROW2(Exception::UnimplementedVirtualMethod, 
-        "ContactGeometryImpl", "calcCurvature()"); }
+        "ContactGeometryImpl", "calcCurvature"); }
 
     // Smooth surfaces only.
     virtual const Function& getImplicitFunction() const
     {   SimTK_THROW2(Exception::UnimplementedVirtualMethod, 
-        "ContactGeometryImpl", "getImplicitFunction()"); }
+        "ContactGeometryImpl", "getImplicitFunction"); }
 
     // Convex surfaces only.
-    virtual Vec3 calcSupportPoint(UnitVec3 direction) const
+    virtual Vec3 calcSupportPoint(const UnitVec3& direction) const
     {   SimTK_THROW2(Exception::UnimplementedVirtualMethod, 
-        "ContactGeometryImpl", "calcSupportPoint()"); }
+        "ContactGeometryImpl", "calcSupportPoint"); }
 
     const OBBTree& getOBBTree() const {return obbTree;}
 
@@ -113,7 +114,14 @@ public:
     Mat33 calcSurfaceHessian(const Vec3& point) const;
     Real  calcGaussianCurvature(const Vec3& gradient,
                                 const Mat33& Hessian) const;
-	Real  calcSurfaceCurvatureInDirection(const Vec3& point, const UnitVec3& direction) const;
+    Real  calcSurfaceCurvatureInDirection(const Vec3& point, 
+                                          const UnitVec3& direction) const;
+    // Generic method for calculating principal curvatures kmax,kmin and
+    // corresponding unit tangent vector directions R_SP.x() and R_SP.y().
+    // R_SP.z() is the surface unit normal at P, with z=x X y.
+    void calcSurfacePrincipalCurvatures(const Vec3& point,
+                                        Vec2& curvature,
+                                        Rotation& R_SP) const;
 
     Vec3 projectDownhillToNearestPoint(const Vec3& Q) const;
 
@@ -441,16 +449,16 @@ public:
     void setOwner(const ContactGeometry::HalfSpace::Impl& owner) {ownerp=&owner;}
 
     // Value is positive for x>0.
-    Real calcValue(const Vector& P) const {return P[0];}
+    Real calcValue(const Vector& P) const override {return P[0];}
     // First derivative w.r.t. x is 1, all else is zero.
     Real calcDerivative(const Array_<int>& components, 
-                        const Vector& P) const
+                        const Vector& P) const override
     {   if (components.empty()) return calcValue(P);
         if (components.size()==1 && components[0]==0) return 1;
         return 0; }
 
-    int getArgumentSize() const {return 3;}
-    int getMaxDerivativeOrder() const
+    int getArgumentSize() const override {return 3;}
+    int getMaxDerivativeOrder() const override
     {   return std::numeric_limits<int>::max(); }
 private:
     const ContactGeometry::HalfSpace::Impl* ownerp; // just a ref.; don't delete
@@ -461,34 +469,34 @@ class ContactGeometry::HalfSpace::Impl : public ContactGeometryImpl {
 public:
     Impl() : ContactGeometryImpl() {
     }
-    ContactGeometryImpl* clone() const {
+    ContactGeometryImpl* clone() const override {
         return new Impl();
     }
 
-    ContactGeometryTypeId getTypeId() const {return classTypeId();}
+    ContactGeometryTypeId getTypeId() const override {return classTypeId();}
 
-    DecorativeGeometry createDecorativeGeometry() const;
+    DecorativeGeometry createDecorativeGeometry() const override;
     Vec3 findNearestPoint(const Vec3& position, bool& inside, 
-                          UnitVec3& normal) const;
+                          UnitVec3& normal) const override;
     bool intersectsRay(const Vec3& origin, const UnitVec3& direction, 
-                       Real& distance, UnitVec3& normal) const;
-    void getBoundingSphere(Vec3& center, Real& radius) const;
+                       Real& distance, UnitVec3& normal) const override;
+    void getBoundingSphere(Vec3& center, Real& radius) const override;
 
-    bool isSmooth() const {return true;}
-    bool isConvex() const {return false;}
-    bool isFinite() const {return false;}
+    bool isSmooth() const override {return true;}
+    bool isConvex() const override {return false;}
+    bool isFinite() const override {return false;}
 
     // Curvature is zero everywhere. Since the half plane occupies x>0 in
     // its own frame, the surface normal is -x, and -x,y,-z forms a right 
     // handed set.
     void calcCurvature(const Vec3& point, Vec2& curvature, 
-                       Rotation& orientation) const
+                       Rotation& orientation) const override
     {   curvature = 0;
         orientation.setRotationFromUnitVecsTrustMe
             (UnitVec3(-XAxis), UnitVec3(YAxis), UnitVec3(-ZAxis));
     }
 
-    const Function& getImplicitFunction() const {return function;}
+    const Function& getImplicitFunction() const override {return function;}
 
     static ContactGeometryTypeId classTypeId() {
         static const ContactGeometryTypeId id = 
@@ -510,11 +518,11 @@ public:
     CylinderImplicitFunction(const ContactGeometry::Cylinder::Impl& owner)
     :   ownerp(&owner) {}
     void setOwner(const ContactGeometry::Cylinder::Impl& owner) {ownerp=&owner;}
-    Real calcValue(const Vector& x) const;
+    Real calcValue(const Vector& x) const override;
     Real calcDerivative(const Array_<int>& derivComponents,
-                        const Vector& x) const;
-    int getArgumentSize() const {return 3;}
-    int getMaxDerivativeOrder() const
+                        const Vector& x) const override;
+    int getArgumentSize() const override {return 3;}
+    int getMaxDerivativeOrder() const override
     {   return std::numeric_limits<int>::max(); }
 private:
     const ContactGeometry::Cylinder::Impl* ownerp; // just a reference; don't delete
@@ -526,7 +534,7 @@ public:
         function.setOwner(*this);
     }
 
-    ContactGeometryImpl* clone() const {
+    ContactGeometryImpl* clone() const override {
         return new Impl(radius);
     }
     Real getRadius() const {
@@ -536,38 +544,42 @@ public:
         radius = r;
     }
 
-    ContactGeometryTypeId getTypeId() const {return classTypeId();}
+    ContactGeometryTypeId getTypeId() const override {return classTypeId();}
 
-    DecorativeGeometry createDecorativeGeometry() const;
+    DecorativeGeometry createDecorativeGeometry() const override;
     Vec3 findNearestPoint(const Vec3& position, bool& inside,
-                          UnitVec3& normal) const;
+                          UnitVec3& normal) const override;
     bool intersectsRay(const Vec3& origin, const UnitVec3& direction,
-                       Real& distance, UnitVec3& normal) const;
-    void getBoundingSphere(Vec3& center, Real& radius) const;
+                       Real& distance, UnitVec3& normal) const override;
+    void getBoundingSphere(Vec3& center, Real& radius) const override;
 
-    bool isSmooth() const {return true;}
-    bool isConvex() const {return true;}
-    bool isFinite() const {return false;}
+    bool isSmooth() const override {return true;} //TODO: only for infinite
+    bool isConvex() const override {return true;}
+    bool isFinite() const override {return false;}
 
-    Vec3 calcSupportPoint(UnitVec3 direction) const {
+    Vec3 calcSupportPoint(const UnitVec3& direction) const override {
         assert(false);
         return Vec3(NaN);
     }
 
     void calcCurvature(const Vec3& point, Vec2& curvature,
-                       Rotation& orientation) const;
+                       Rotation& orientation) const override;
 
-    virtual void shootGeodesicInDirectionUntilLengthReachedAnalytical(const Vec3& xP, const UnitVec3& tP,
-            const Real& terminatingLength, const GeodesicOptions& options, Geodesic& geod) const;
+    void shootGeodesicInDirectionUntilLengthReachedAnalytical
+       (const Vec3& xP, const UnitVec3& tP,
+        const Real& terminatingLength, const GeodesicOptions& options, 
+        Geodesic& geod) const override;
 
-    virtual void shootGeodesicInDirectionUntilPlaneHitAnalytical(const Vec3& xP, const UnitVec3& tP,
-            const Plane& terminatingPlane, const GeodesicOptions& options,
-            Geodesic& geod) const;
+    void shootGeodesicInDirectionUntilPlaneHitAnalytical
+       (const Vec3& xP, const UnitVec3& tP,
+        const Plane& terminatingPlane, const GeodesicOptions& options,
+        Geodesic& geod) const override;
 
-    virtual void calcGeodesicAnalytical(const Vec3& xP, const Vec3& xQ,
-                const Vec3& tPhint, const Vec3& tQhint, Geodesic& geod) const;
+    void calcGeodesicAnalytical
+       (const Vec3& xP, const Vec3& xQ, const Vec3& tPhint, const Vec3& tQhint, 
+        Geodesic& geod) const override;
 
-    const Function& getImplicitFunction() const {
+    const Function& getImplicitFunction() const override {
         return function;
     }
 
@@ -592,11 +604,11 @@ public:
     SphereImplicitFunction(const ContactGeometry::Sphere::Impl& owner) 
     :   ownerp(&owner) {}
     void setOwner(const ContactGeometry::Sphere::Impl& owner) {ownerp=&owner;}
-    Real calcValue(const Vector& x) const;
+    Real calcValue(const Vector& x) const override;
     Real calcDerivative(const Array_<int>& derivComponents, 
-                        const Vector& x) const;
-    int getArgumentSize() const {return 3;}
-    int getMaxDerivativeOrder() const
+                        const Vector& x) const override;
+    int getArgumentSize() const override {return 3;}
+    int getMaxDerivativeOrder() const override
     {   return std::numeric_limits<int>::max(); }
 private:
     const ContactGeometry::Sphere::Impl* ownerp; // just a reference; don't delete
@@ -609,7 +621,7 @@ public:
         createOBBTree(); 
     }
 
-    ContactGeometryImpl* clone() const {
+    ContactGeometryImpl* clone() const override {
         return new Impl(radius);
     }
     Real getRadius() const {
@@ -619,36 +631,38 @@ public:
         radius = r;
     }
 
-    ContactGeometryTypeId getTypeId() const {return classTypeId();}
+    ContactGeometryTypeId getTypeId() const override {return classTypeId();}
 
-    DecorativeGeometry createDecorativeGeometry() const;
+    DecorativeGeometry createDecorativeGeometry() const override;
     Vec3 findNearestPoint(const Vec3& position, bool& inside, 
-                          UnitVec3& normal) const;
+                          UnitVec3& normal) const override;
     bool intersectsRay(const Vec3& origin, const UnitVec3& direction, 
-                       Real& distance, UnitVec3& normal) const;
-    void getBoundingSphere(Vec3& center, Real& radius) const;
+                       Real& distance, UnitVec3& normal) const override;
+    void getBoundingSphere(Vec3& center, Real& radius) const override;
 
-    bool isSmooth() const {return true;}
-    bool isConvex() const {return true;}
-    bool isFinite() const {return true;}
+    bool isSmooth() const override {return true;}
+    bool isConvex() const override {return true;}
+    bool isFinite() const override {return true;}
 
-    Vec3 calcSupportPoint(UnitVec3 direction) const {
+    Vec3 calcSupportPoint(const UnitVec3& direction) const override {
         return radius*direction;
     }
     void calcCurvature(const Vec3& point, Vec2& curvature, 
-                       Rotation& orientation) const;
+                       Rotation& orientation) const override;
 
-    virtual void shootGeodesicInDirectionUntilLengthReachedAnalytical(const Vec3& xP, const UnitVec3& tP,
-            const Real& terminatingLength, const GeodesicOptions& options, Geodesic& geod) const;
+    void shootGeodesicInDirectionUntilLengthReachedAnalytical
+       (const Vec3& xP, const UnitVec3& tP, const Real& terminatingLength, 
+        const GeodesicOptions& options, Geodesic& geod) const override;
 
-    virtual void shootGeodesicInDirectionUntilPlaneHitAnalytical(const Vec3& xP, const UnitVec3& tP,
-            const Plane& terminatingPlane, const GeodesicOptions& options,
-            Geodesic& geod) const;
+    void shootGeodesicInDirectionUntilPlaneHitAnalytical
+       (const Vec3& xP, const UnitVec3& tP, const Plane& terminatingPlane, 
+        const GeodesicOptions& options, Geodesic& geod) const override;
 
-    virtual void calcGeodesicAnalytical(const Vec3& xP, const Vec3& xQ,
-                const Vec3& tPhint, const Vec3& tQhint, Geodesic& geod) const;
+    void calcGeodesicAnalytical
+       (const Vec3& xP, const Vec3& xQ, const Vec3& tPhint, const Vec3& tQhint, 
+        Geodesic& geod) const override;
 
-    const Function& getImplicitFunction() const {
+    const Function& getImplicitFunction() const override {
         return function;
     }
 
@@ -675,11 +689,11 @@ public:
     EllipsoidImplicitFunction(const ContactGeometry::Ellipsoid::Impl& owner) 
     :   ownerp(&owner) {}
     void setOwner(const ContactGeometry::Ellipsoid::Impl& owner) {ownerp=&owner;}
-    Real calcValue(const Vector& x) const;
+    Real calcValue(const Vector& x) const override;
     Real calcDerivative(const Array_<int>& derivComponents, 
-                        const Vector& x) const;
-    int getArgumentSize() const {return 3;}
-    int getMaxDerivativeOrder() const
+                        const Vector& x) const override;
+    int getArgumentSize() const override {return 3;}
+    int getMaxDerivativeOrder() const override
     {   return std::numeric_limits<int>::max(); }
 private:
     const ContactGeometry::Ellipsoid::Impl* ownerp;// just a ref.; don't delete
@@ -693,7 +707,7 @@ public:
     {   function.setOwner(*this);
         createOBBTree(); }
 
-    ContactGeometryImpl* clone() const {return new Impl(radii);}
+    ContactGeometryImpl* clone() const override {return new Impl(radii);}
     const Vec3& getRadii() const {return radii;}
     void setRadii(const Vec3& r) 
     {   radii = r; curvatures = Vec3(1/r[0],1/r[1],1/r[2]); }
@@ -712,27 +726,27 @@ public:
     void findParaboloidAtPointWithNormal(const Vec3& Q, const UnitVec3& n,
         Transform& X_EP, Vec2& k) const;
 
-    ContactGeometryTypeId getTypeId() const {return classTypeId();}
+    ContactGeometryTypeId getTypeId() const override {return classTypeId();}
 
-    DecorativeGeometry createDecorativeGeometry() const;
+    DecorativeGeometry createDecorativeGeometry() const override;
     Vec3 findNearestPoint(const Vec3& position, bool& inside, 
-                          UnitVec3& normal) const;
+                          UnitVec3& normal) const override;
     bool intersectsRay(const Vec3& origin, const UnitVec3& direction, 
-                       Real& distance, UnitVec3& normal) const;
-    void getBoundingSphere(Vec3& center, Real& radius) const;
+                       Real& distance, UnitVec3& normal) const override;
+    void getBoundingSphere(Vec3& center, Real& radius) const override;
 
-    bool isSmooth() const {return true;}
-    bool isConvex() const {return true;}
-    bool isFinite() const {return true;}
+    bool isSmooth() const override {return true;}
+    bool isConvex() const override {return true;}
+    bool isFinite() const override {return true;}
 
     // The point furthest in this direction is the unique point whose outward
     // normal is this direction.
-    Vec3 calcSupportPoint(UnitVec3 direction) const {
+    Vec3 calcSupportPoint(const UnitVec3& direction) const override {
         return findPointWithThisUnitNormal(direction);
     }
     void calcCurvature(const Vec3& point, Vec2& curvature, 
-                       Rotation& orientation) const;
-    const Function& getImplicitFunction() const {
+                       Rotation& orientation) const override;
+    const Function& getImplicitFunction() const override {
         return function;
     }
 
@@ -820,11 +834,11 @@ public:
     :   ownerp(&owner) {}
     void setOwner(const ContactGeometry::SmoothHeightMap::Impl& owner) 
     {   ownerp=&owner; }
-    Real calcValue(const Vector& x) const;
+    Real calcValue(const Vector& x) const override;
     Real calcDerivative(const Array_<int>& derivComponents, 
-                        const Vector& x) const;
-    int getArgumentSize() const {return 3;}
-    int getMaxDerivativeOrder() const
+                        const Vector& x) const override;
+    int getArgumentSize() const override {return 3;}
+    int getMaxDerivativeOrder() const override
     {   return std::numeric_limits<int>::max(); }
 private:
     // just a reference; don't delete
@@ -837,32 +851,32 @@ class ContactGeometry::SmoothHeightMap::Impl : public ContactGeometryImpl {
 public:
     explicit Impl(const BicubicSurface& surface);
 
-    ContactGeometryImpl* clone() const {
+    ContactGeometryImpl* clone() const override {
         return new Impl(surface);
     }
 
     const BicubicSurface& getBicubicSurface() const {return surface;}
     BicubicSurface::PatchHint& updHint() const {return hint;}
 
-    ContactGeometryTypeId getTypeId() const {return classTypeId();}
+    ContactGeometryTypeId getTypeId() const override {return classTypeId();}
 
-    DecorativeGeometry createDecorativeGeometry() const;
+    DecorativeGeometry createDecorativeGeometry() const override;
     Vec3 findNearestPoint(const Vec3& position, bool& inside, 
-                          UnitVec3& normal) const;
+                          UnitVec3& normal) const override;
 
     bool intersectsRay(const Vec3& origin, const UnitVec3& direction, 
-                       Real& distance, UnitVec3& normal) const;
+                       Real& distance, UnitVec3& normal) const override;
 
-    void getBoundingSphere(Vec3& center, Real& radius) const {
+    void getBoundingSphere(Vec3& center, Real& radius) const override {
         center = boundingSphere.getCenter();
         radius = boundingSphere.getRadius();
     }
 
-    bool isSmooth() const {return true;}
-    bool isConvex() const {return false;}
-    bool isFinite() const {return true;}
+    bool isSmooth() const override {return true;}
+    bool isConvex() const override {return false;}
+    bool isFinite() const override {return true;}
 
-    Vec3 calcSupportPoint(UnitVec3 direction) const {
+    Vec3 calcSupportPoint(const UnitVec3& direction) const override {
         assert(false);
         return Vec3(NaN);
     }
@@ -870,13 +884,14 @@ public:
     // We ignore the z coordinate here and just return the curvature of
     // the unique point at (x,y).
     void calcCurvature(const Vec3& point, Vec2& curvature, 
-                       Rotation& orientation) const {
+                       Rotation& orientation) const override {
         Transform X_SP;
         surface.calcParaboloid(Vec2(point[0],point[1]), hint, X_SP, curvature);
         orientation = X_SP.R();
     }
 
-    const Function& getImplicitFunction() const {return implicitFunction;}
+    const Function& getImplicitFunction() const override 
+    {   return implicitFunction; }
 
     static ContactGeometryTypeId classTypeId() {
         static const ContactGeometryTypeId id = 
@@ -910,12 +925,64 @@ private:
 
 
 
+
+//==============================================================================
+//                                BRICK IMPL
+//==============================================================================
+
+class ContactGeometry::Brick::Impl : public ContactGeometryImpl {
+public:
+    explicit Impl(const Vec3& halfLengths) : m_box(halfLengths) {
+        createOBBTree(); 
+    }
+
+    ContactGeometryImpl* clone() const override {
+        return new Impl(getHalfLengths());
+    }
+    const Vec3& getHalfLengths() const {return m_box.getHalfLengths();}
+
+    void setHalfLengths(const Vec3& halfLengths) {
+        m_box.setHalfLengths(halfLengths);
+    }
+
+    const Geo::Box& getGeoBox() const {return m_box;}
+
+    ContactGeometryTypeId getTypeId() const override {return classTypeId();}
+
+    DecorativeGeometry createDecorativeGeometry() const override;
+    Vec3 findNearestPoint(const Vec3& position, bool& inside, 
+                          UnitVec3& normal) const override;
+    bool intersectsRay(const Vec3& origin, const UnitVec3& direction, 
+                       Real& distance, UnitVec3& normal) const override;
+    void getBoundingSphere(Vec3& center, Real& radius) const override;
+
+    bool isSmooth() const override {return false;}
+    bool isConvex() const override {return true;}
+    bool isFinite() const override {return true;}
+
+    Vec3 calcSupportPoint(const UnitVec3& direction) const override {
+        return m_box.findSupportPoint(direction);
+    }
+
+    static ContactGeometryTypeId classTypeId() {
+        static const ContactGeometryTypeId id = 
+            createNewContactGeometryTypeId();
+        return id;
+    }
+private:
+    void createOBBTree();
+
+    Geo::Box        m_box;
+};
+
+
+
 //==============================================================================
 //                            OBB TREE NODE IMPL
 //==============================================================================
 class OBBTreeNodeImpl {
 public:
-    OBBTreeNodeImpl() : child1(NULL), child2(NULL) {
+    OBBTreeNodeImpl() : child1(NULL), child2(NULL), numTriangles(0) {
     }
     OBBTreeNodeImpl(const OBBTreeNodeImpl& copy);
     ~OBBTreeNodeImpl();
@@ -946,32 +1013,33 @@ public:
     Impl(const ArrayViewConst_<Vec3>& vertexPositions, 
          const ArrayViewConst_<int>& faceIndices, bool smooth);
     Impl(const PolygonalMesh& mesh, bool smooth);
-    ContactGeometryImpl* clone() const {
+    ContactGeometryImpl* clone() const override {
         return new Impl(*this);
     }
 
-    ContactGeometryTypeId getTypeId() const {return classTypeId();}
+    ContactGeometryTypeId getTypeId() const override {return classTypeId();}
 
-    DecorativeGeometry createDecorativeGeometry() const;
     Vec3     findPoint(int face, const Vec2& uv) const;
     Vec3     findCentroid(int face) const;
     UnitVec3 findNormalAtPoint(int face, const Vec2& uv) const;
-    Vec3 findNearestPoint(const Vec3& position, bool& inside, 
-                          UnitVec3& normal) const;
     Vec3 findNearestPoint(const Vec3& position, bool& inside, int& face, 
                           Vec2& uv) const;
     Vec3 findNearestPointToFace(const Vec3& position, int face, Vec2& uv) const;
+    void createPolygonalMesh(PolygonalMesh& mesh) const;
+
+    DecorativeGeometry createDecorativeGeometry() const override;
+    Vec3 findNearestPoint(const Vec3& position, bool& inside, 
+                          UnitVec3& normal) const override;
     bool intersectsRay(const Vec3& origin, const UnitVec3& direction, 
-                       Real& distance, UnitVec3& normal) const;
+                       Real& distance, UnitVec3& normal) const override;
     bool intersectsRay(const Vec3& origin, const UnitVec3& direction, 
                        Real& distance, int& face, Vec2& uv) const;
-    void getBoundingSphere(Vec3& center, Real& radius) const;
+    void getBoundingSphere(Vec3& center, Real& radius) const override;
 
-    bool isSmooth() const {return false;}
-    bool isConvex() const {return false;}
-    bool isFinite() const {return true;}
+    bool isSmooth() const override {return false;}
+    bool isConvex() const override {return false;}
+    bool isFinite() const override {return true;}
 
-    void createPolygonalMesh(PolygonalMesh& mesh) const;
 
     static ContactGeometryTypeId classTypeId() {
         static const ContactGeometryTypeId id = 
@@ -1028,6 +1096,7 @@ public:
         vertices[0] = vert1;
         vertices[1] = vert2;
         vertices[2] = vert3;
+        edges[0] = edges[1] = edges[2] = -1; // filled in later
     }
     int         vertices[3];
     int         edges[3];
@@ -1060,11 +1129,11 @@ public:
     TorusImplicitFunction(const ContactGeometry::Torus::Impl& owner)
     :   ownerp(&owner) {}
     void setOwner(const ContactGeometry::Torus::Impl& owner) {ownerp=&owner;}
-    Real calcValue(const Vector& x) const;
+    Real calcValue(const Vector& x) const override;
     Real calcDerivative(const Array_<int>& derivComponents,
-                        const Vector& x) const;
-    int getArgumentSize() const {return 3;}
-    int getMaxDerivativeOrder() const
+                        const Vector& x) const override;
+    int getArgumentSize() const override {return 3;}
+    int getMaxDerivativeOrder() const override
     {   return std::numeric_limits<int>::max(); }
 private:
     const ContactGeometry::Torus::Impl* ownerp; // just a reference; don't delete
@@ -1077,7 +1146,7 @@ public:
         function.setOwner(*this);
     }
 
-    ContactGeometryImpl* clone() const {
+    ContactGeometryImpl* clone() const override {
         return new Impl(torusRadius, tubeRadius);
     }
     Real getTorusRadius() const {
@@ -1093,26 +1162,26 @@ public:
         tubeRadius = r;
     }
 
-    ContactGeometryTypeId getTypeId() const {return classTypeId();}
+    ContactGeometryTypeId getTypeId() const override {return classTypeId();}
 
-    DecorativeGeometry createDecorativeGeometry() const;
+    DecorativeGeometry createDecorativeGeometry() const override;
     bool intersectsRay(const Vec3& origin, const UnitVec3& direction,
-                       Real& distance, UnitVec3& normal) const;
-    void getBoundingSphere(Vec3& center, Real& radius) const;
+                       Real& distance, UnitVec3& normal) const override;
+    void getBoundingSphere(Vec3& center, Real& radius) const override;
 
     void createPolygonalMesh(PolygonalMesh& mesh) const;
 
-    bool isSmooth() const {return true;}
-    bool isConvex() const {return false;}
-    bool isFinite() const {return true;}
+    bool isSmooth() const override {return true;}
+    bool isConvex() const override {return false;}
+    bool isFinite() const override {return true;}
 
-    Vec3 calcSupportPoint(UnitVec3 direction) const;
+    Vec3 calcSupportPoint(const UnitVec3& direction) const override;
 
     void calcCurvature(const Vec3& point, Vec2& curvature,
-                       Rotation& orientation) const;
+                       Rotation& orientation) const override;
 
     Vec3 findNearestPoint(const Vec3& position, bool& inside,
-            UnitVec3& normal) const;
+            UnitVec3& normal) const override;
 
 //    TODO
 //    virtual void shootGeodesicInDirectionUntilLengthReachedAnalytical(const Vec3& xP, const UnitVec3& tP,
@@ -1125,7 +1194,7 @@ public:
 //    virtual void calcGeodesicAnalytical(const Vec3& xP, const Vec3& xQ,
 //                const Vec3& tPhint, const Vec3& tQhint, Geodesic& geod) const;
 
-    const Function& getImplicitFunction() const {
+    const Function& getImplicitFunction() const override {
         return function;
     }
 
