@@ -17,7 +17,7 @@
 
 #include <vector>
 
-namespace Ipopt
+namespace SimTKIpopt
 {
   /* forward declarations */
   class VectorSpace;
@@ -728,10 +728,10 @@ namespace Ipopt
    if (dbg_jrnl.Verbosity() >= (__verbose_level)) { \
       if (dbg_jrnl.Jnlst()!=NULL) { \
         (__vec).Print(dbg_jrnl.Jnlst(), \
-		      J_ERROR, J_DBG, \
-		      __vec_name, \
-		      dbg_jrnl.IndentationLevel()*2, \
-		      "# "); \
+              J_ERROR, J_DBG, \
+              __vec_name, \
+              dbg_jrnl.IndentationLevel()*2, \
+              "# "); \
       } \
    }
 #endif // IP_DEBUG
