@@ -6,7 +6,7 @@
  * Biological Structures at Stanford, funded under the NIH Roadmap for        *
  * Medical Research, grant U54 GM072970. See https://simtk.org/home/simbody.  *
  *                                                                            *
- * Portions copyright (c) 2013 Stanford University and the Authors.           *
+ * Portions copyright (c) 2013-4 Stanford University and the Authors.         *
  * Authors: Michael Sherman                                                   *
  * Contributors: Kevin He                                                     *
  *                                                                            *
@@ -29,6 +29,7 @@
 #include "simmath/MultibodyGraphMaker.h"
 
 #include <exception>
+#include <set>
 #include <stdexcept>
 #include <string>
 #include <iostream>
@@ -36,6 +37,10 @@
 
 using std::cout; using std::endl;
 
+#ifdef _MSC_VER
+#pragma warning(disable:4996) // don't warn about strcat, sprintf, etc.
+#endif
+
 namespace SimTK {
 
 //------------------------------------------------------------------------------
@@ -257,18 +262,34 @@ void MultibodyGraphMaker::generateGraph() {
     //
     // While we're at it, look for dangling massless bodies -- they are only
     // allowed if they were originally connected to at least two other bodies
-    // by given tree-eligible joints.
+    // by given tree-eligible joints, or if they are connected by a weld 
+    // joint and thus have no mobility.
     // TODO: "must be loop joint" joints shouldn't count but we're not
     // checking.
     for (int bn=1; bn < getNumBodies(); ++bn) { // skip Ground
         const Body& body = getBody(bn);
-        const int nJoints = // how many joints connect to this body?
-            (int)(body.jointsAsChild.size()+body.jointsAsParent.size());
-        if (body.mass == 0 && nJoints < 2)
+        const int nJoints = body.getNumJoints();
+        if (body.mass == 0) {
+            if (nJoints == 0) {
                 throw std::runtime_error(
                     "generateGraph(): body " + body.name + 
-                    " is massless but connected by fewer than 2 joints.");
-
+                    " is massless but free (no joint). Must be internal or"
+                    " welded to a massful body.");
+            }
+            if (nJoints == 1) {
+                const int jnum = body.jointsAsChild.empty() 
+                                    ? body.jointsAsParent[0] 
+                                    : body.jointsAsChild[0];
+                const Joint& joint = getJoint(jnum);
+                const JointType& jtype = getJointType(joint.jointTypeNum);
+                if (jtype.numMobilities > 0) {
+                    throw std::runtime_error(
+                        "generateGraph(): body " + body.name + 
+                        " is massless but not internal and not welded to"
+                        " a massful body.");
+                }                               
+            }
+        }
         if (nJoints==0 || (body.mustBeBaseBody && !bodiesAreConnected(bn, 0))) 
             connectBodyToGround(bn);
     }
@@ -581,23 +602,38 @@ findHeaviestUnassignedReverseJoint(int inboardBody) const {
 // previously-disconnected base body to Ground and then call this again.
 //
 // We violate the breadth-first heuristic to avoid ending a branch with a 
-// massless body. If we add a massless body, we'll keep going out along its
-// branch until we hit a massful body. It is a disaster if we fail to find a
-// massful body because a tree that terminates in a massless body will generate
-// a singular mass matrix. We'll throw an exception in that case, but note that
-// this may just be a failure of the heuristic; there may be some tree that 
-// could have avoided the terminal massless body but we failed to discover it.
+// massless body, unless it is welded to its parent. If we add a mobile massless
+// body, we'll keep going out along its branch until we hit a massful body. It 
+// is a disaster if we fail to find a massful body because a tree that 
+// terminates in a mobile massless body will generate a singular mass matrix. 
+// We'll throw an exception in that case, but note that this may just be a 
+// failure of the heuristic; there may be some tree that could have avoided the 
+// terminal massless body but we failed to discover it.
 //
 // TODO: keep a list of unprocessed joints so we don't have to go through
 // all of them again at each level.
 void MultibodyGraphMaker::growTree() {
+    // Record the joints for which we added mobilizers during this subtree
+    // sweep. That way if we jumped levels ahead due to massless bodies we
+    // can take credit and keep going rather than quit.
+    std::set<int> jointsAdded;
     for (int level=1; ;++level) { // level of outboard (mobilized) body
         bool anyMobilizerAdded = false;
         for (int jNum=0; jNum<getNumJoints(); ++jNum) {
             // See if this joint is at the right level (meaning its inboard
             // body is at level-1) and is available to become a mobilizer.
             const Joint& joint = getJoint(jNum);
-            if (joint.hasMobilizer()) continue; // already done
+            if (joint.hasMobilizer()) {
+                // Already done -- one of ours?
+                if (jointsAdded.count(jNum)) {
+                    // We added it during this growTree() call -- is it at
+                    // the current level?
+                    const Mobilizer& mob = mobilizers[joint.mobilizer];
+                    if (mob.getLevel() == level)
+                        anyMobilizerAdded = true; // Added one at level.
+                }
+                continue; 
+            }
             if (joint.mustBeLoopJoint) continue; // can't be a tree joint
             const Body& parent = getBody(joint.parentBodyNum);
             const Body& child  = getBody(joint.childBodyNum);
@@ -609,12 +645,16 @@ void MultibodyGraphMaker::growTree() {
                 if (child.level != level-1) continue; // not time yet
             } 
             addMobilizerForJoint(jNum);
+            jointsAdded.insert(jNum);
             anyMobilizerAdded = true;
 
             // We just made joint jNum a mobilizer. If its outboard body 
-            // is massless, we need to keep extending this branch of the tree
-            // until we can end it with a massful body.
-            if (getBody(mobilizers.back().outboardBody).mass > 0)
+            // is massless and the mobilizer was not a weld, we need to keep 
+            // extending this branch of the tree until we can end it with a 
+            // massful body.
+            const Body& outboard = getBody(mobilizers.back().outboardBody);
+            const JointType& jtype = getJointType(joint.jointTypeNum);
+            if (jtype.numMobilities == 0 || outboard.mass > 0)
                 continue;
 
             // Pick a further-outboard body with mass and extend branch to it.
@@ -626,11 +666,13 @@ void MultibodyGraphMaker::growTree() {
                 const int jfwd = findHeaviestUnassignedForwardJoint(bNum);
                 if (jfwd>=0 && getBody(getJoint(jfwd).childBodyNum).mass > 0) {
                     addMobilizerForJoint(jfwd);
+                    jointsAdded.insert(jfwd);
                     break;
                 }
                 const int jrev = findHeaviestUnassignedReverseJoint(bNum);
                 if (jrev>=0 && getBody(getJoint(jrev).parentBodyNum).mass > 0) {
                     addMobilizerForJoint(jrev);
+                    jointsAdded.insert(jrev);
                     break;
                 }
 
@@ -638,10 +680,12 @@ void MultibodyGraphMaker::growTree() {
                 // body (if there is one) and keep trying.
                 if (jfwd >= 0) {
                     addMobilizerForJoint(jfwd);
+                    jointsAdded.insert(jfwd);
                     continue;
                 }
                 if (jrev >= 0) {
                     addMobilizerForJoint(jrev);
+                    jointsAdded.insert(jrev);
                     continue;
                 }
 
